/**
* High Availability Service (HA) for JOnAS
*
* Copyright (C) 2007,2008 Bull S.A.S.
* Copyright (C) 2006 Distributed Systems Lab.
* Universidad Polit??cnica de Madrid (Spain)
* Contact: http://lsd.ls.fi.upm.es/lsd
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id: HAClientInterceptor.java 2438 2010-02-24 16:20:30Z benoitf $
* --------------------------------------------------------------------------
*/

package org.ow2.cmi.ha.interceptor.jrmp;

import java.io.IOException;
import java.util.Stack;

import org.ow2.carol.rmi.interceptor.api.JClientRequestInfo;
import org.ow2.carol.rmi.interceptor.spi.JClientRequestInterceptor;
import org.ow2.cmi.ha.RequestId;
import org.ow2.cmi.ha.interceptor.HAContext;
import org.ow2.cmi.ha.interceptor.HACurrent;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * This class represents the client side of the interceptor mechanism
 * implemented to transfer replication related information between client
 * and server sides.
 */
public class HAClientInterceptor implements JClientRequestInterceptor {


    /**
     * Id for serializable class.
     */
    private static final long serialVersionUID = 7391445843768730939L;

    /**
     * Interceptor name
     */
    private String interceptorName = "HAClientInterceptor";

    /**
     * Logger used for traces
     */
    private static Log logger = LogFactory.getLog(HAClientInterceptor.class);

    /**
     * Constructor
     */
    public HAClientInterceptor() {
    }

    /**
     * Get the name of this interceptor
     * @return name
     */
    public String name() {
        return interceptorName;
    }

    /**
     * Constructs the HAContext with the required replication information on
     * the client side
     */
    public void sendRequest(final JClientRequestInfo jri) throws IOException {

        // Get the current HA Context info associated to the thread
        HACurrent current = HACurrent.getHACurrent();
        // Fill the "req_id" field to propagate through the interceptor if the
        // nextReq has been included in the HA current context (held in the
        // thread-local variable) by the stub

        HAContext newHACtx = new HAContext();

        Stack<RequestId> requests = current.getRequests();
        newHACtx.setRequests(requests);
        newHACtx.setOnFailover(current.isOnFailover());
        logger.debug("Propagating requests chain: " + newHACtx);

        // Create the new HAContext to propagate between interceptors
        HAServiceContext haServiceContext = new HAServiceContext(newHACtx);

        // Finally, include the new HAContext created to be propagated through
        // the interceptor
        jri.addRequestServiceContext(haServiceContext);
    }

    public void receiveReply(final JClientRequestInfo jri) throws IOException {
        getHAContext(jri);
    }

    public void receiveException(final JClientRequestInfo jri) throws IOException {
        getHAContext(jri);
    }

    public void receiveOther(final JClientRequestInfo jri) throws IOException {
        getHAContext(jri);
    }

    //-------------------------------------------------------------------------
    // Other public unused methods
    //-------------------------------------------------------------------------

    public void sendPoll(final JClientRequestInfo jri) throws IOException {
    }

    private void getHAContext(final JClientRequestInfo jri) {
        // Get the HAContext from the received info
        HAServiceContext receivedCtx = (HAServiceContext) jri.getRequestServiceContext(HAServiceContext.HA_CTX_ID);

        if(receivedCtx == null) {
            return;
        }

        logger.debug("receivedCtx: " + receivedCtx);
        // Get the current Thread/HAContext association class
        HACurrent current = HACurrent.getHACurrent();

        Stack<RequestId> requests =
            ((HAContext) receivedCtx.getContextData()).getRequests();
        current.setRequests(requests);
    }
}
