/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2007,2008 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: HAInterceptorInitializer.java 2438 2010-02-24 16:20:30Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.cmi.ha.interceptor.iiop;

import org.omg.CORBA.LocalObject;
import org.omg.PortableInterceptor.ORBInitInfo;
import org.omg.PortableInterceptor.ORBInitializer;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * This class initializes the interception mechanism with iiop.
 * @author Loris Bouzonnet
 */
public class HAInterceptorInitializer extends LocalObject implements ORBInitializer {

    /**
     * Id for serializable class.
     */
    private static final long serialVersionUID = -8881600159403181179L;

    /**
     * Logger for traces
     */
    private static Log logger = LogFactory.getLog(HAInterceptorInitializer.class);

    public void pre_init(final ORBInitInfo info) {
        try {
            info.add_client_request_interceptor(new HAClientInterceptor());
            info.add_server_request_interceptor(new HAServerInterceptor());
        } catch (Exception e) {
            logger.error(
                    "Error while adding the request interceptors", e);
        }
    }

    public void post_init(final ORBInitInfo info) {
    }

}
